@@ -103,9 +103,22 @@ class ANSIColorTest < Test::Unit::TestCase
 
   def test_attributes
     foo = 'foo'
-    for a in Term::ANSIColor.attributes
-      # skip clear for Ruby 1.9 which implements String#clear to empty the string
-      if a != :clear || Term::ANSIColor.support?(:clear)
+    attributes = %i[
+      clear reset bold dark faint italic underline underscore
+      blink rapid_blink reverse negative concealed conceal strikethrough black
+      red green yellow blue magenta cyan white on_black on_red on_green on_yellow
+      on_blue on_magenta on_cyan on_white intense_black bright_black intense_red
+      bright_red intense_green bright_green intense_yellow bright_yellow
+      intense_blue bright_blue intense_magenta bright_magenta intense_cyan
+      bright_cyan intense_white bright_white on_intense_black on_bright_black
+      on_intense_red on_bright_red on_intense_green on_bright_green
+      on_intense_yellow on_bright_yellow on_intense_blue on_bright_blue
+      on_intense_magenta on_bright_magenta on_intense_cyan on_bright_cyan
+      on_intense_white on_bright_white
+    ]
+    for a in attributes
+      # skip :clear and :reverse b/c Ruby implements them on string
+      if a != :clear && a != :reverse
         refute_equal foo, foo_colored = foo.__send__(a)
         assert_equal foo, foo_colored.uncolor
       end
@@ -121,12 +134,68 @@ class ANSIColorTest < Test::Unit::TestCase
     assert_equal Term::ANSIColor.attributes, 'foo'.attributes
   end
 
+  def test_underline_invalid_type
+    assert_raises(ArgumentError) { Term::ANSIColor.underline(type: :nix) { 'foo' } }
+  end
+
+  def test_underline_invalid_color
+    assert_raises(ArgumentError) { Term::ANSIColor.underline(color: :nix) { 'foo' } }
+  end
+
+  def test_underline
+    foo = 'foo'
+    assert_equal "\e[4mfoo\e[0m", Term::ANSIColor.underline { foo }
+    assert_equal "\e[4m\e[58;2;255;135;95mfoo\e[0m", Term::ANSIColor.underline(color: '#ff8040') { foo }
+  end
+
+  def test_underline_double
+    foo = 'foo'
+    assert_equal "\e[4:2mfoo\e[0m", Term::ANSIColor.underline(type: :double) { foo }
+    assert_equal "\e[4:2m\e[58;2;255;135;95mfoo\e[0m", Term::ANSIColor.underline(type: :double, color: '#ff8040') { foo }
+  end
+
+  def test_underline_curly
+    foo = 'foo'
+    assert_equal "\e[4:3mfoo\e[0m", Term::ANSIColor.underline(type: :curly) { foo }
+    assert_equal "\e[4:3m\e[58;2;255;135;95mfoo\e[0m", Term::ANSIColor.underline(type: :curly, color: '#ff8040') { foo }
+  end
+
+  def test_underline_dotted
+    foo = 'foo'
+    assert_equal "\e[4:4mfoo\e[0m", Term::ANSIColor.underline(type: :dotted) { foo }
+    assert_equal "\e[4:4m\e[58;2;255;135;95mfoo\e[0m", Term::ANSIColor.underline(type: :dotted, color: '#ff8040') { foo }
+  end
+
+  def test_underline_dashed
+    foo = 'foo'
+    assert_equal "\e[4:5mfoo\e[0m", Term::ANSIColor.underline(type: :dashed) { foo }
+    assert_equal "\e[4:5m\e[58;2;255;135;95mfoo\e[0m", Term::ANSIColor.underline(type: :dashed, color: '#ff8040') { foo }
+  end
+
+  def test_move_to
+    string_23_23 = "\e[23;23Hred"
+    assert_equal string_23_23, string.move_to(23, 23)
+    assert_equal string_23_23, Color.move_to(23, 23, string)
+    assert_equal string_23_23, Color.move_to(23, 23) { string }
+    assert_equal string_23_23, Term::ANSIColor.move_to(23, 23) { string }
+    assert_equal string_23_23, move_to(23, 23) { string }
+  end
+
+  def test_return_to_position
+    string_return = "\e[sred\e[u"
+    assert_equal string_return, string.return_to_position
+    assert_equal string_return, Color.return_to_position(string)
+    assert_equal string_return, Color.return_to_position { string }
+    assert_equal string_return, Term::ANSIColor.return_to_position { string }
+    assert_equal string_return, return_to_position { string }
+  end
+
   def test_coloring_string_like
     assert_equal "\e[31mred\e[0m", red(string_like)
   end
 
   def test_frozen
-    string = 'foo'
+    string = 'foo'.dup
     red = string.red
     string.extend(Term::ANSIColor).freeze
     assert string.frozen?
@@ -139,4 +208,20 @@ class ANSIColorTest < Test::Unit::TestCase
     string = Color.red(string)
     assert_kind_of Term::ANSIColor, 'new'
   end
+
+  def test_coloring
+    assert Term::ANSIColor.coloring?
+    Term::ANSIColor.coloring = false
+    assert_false Term::ANSIColor.coloring?
+  ensure
+    Term::ANSIColor.coloring = true
+  end
+
+  def test_true_coloring
+    assert_false Term::ANSIColor.true_coloring?
+    Term::ANSIColor.true_coloring = true
+    assert Term::ANSIColor.true_coloring?
+  ensure
+    Term::ANSIColor.true_coloring = false
+  end
 end
